/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package balabit;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.Scanner;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import weka.core.Instance;
import weka.core.Instances;

/**
 *
 * @author manyi 
 * INPUT: balabit_training_type<TYPEID>.csv with the following structure: ... class, session, n_from, n_to 
 * INPUT: the header.txt contains the header for the output arff files
 *
 * OUTPUT: 10 arff files for the 10 users, user<i>_test.arff test files for the
 * binary classifiers the last attribute: @attribute userid {0,1}
 */
public class MainBalabitTrainSplit {

    private Map<String, MyInteger> frequency = new TreeMap<>();
    public int[] userids = {7, 9, 12, 15, 16, 20, 21, 23, 29, 35};
    private ArrayList<String> header = readHeader("header.txt");
    private String trainfile = "balabit_training_type4.csv";

    public static void main(String[] args) {
        MainBalabitTrainSplit obj = new MainBalabitTrainSplit();
        obj.countClassInstances();
        obj.splitTrainingFile();
    }

    private void countClassInstances() {
        Scanner scanner = null;
        try {
            scanner = new Scanner(new File(trainfile));
        } catch (FileNotFoundException ex) {
            Logger.getLogger(MainBalabitTrainSplit.class.getName()).log(Level.SEVERE, null, ex);
        }
        // ignore header
        scanner.nextLine();
        while (scanner.hasNextLine()) {
            String line = scanner.nextLine();
            String items[] = line.split(",");
            int n = items.length;
            String classid = items[n - 4].trim();
            MyInteger counter = frequency.get(classid);
            if (counter == null) {
                frequency.put(classid, new MyInteger());
            } else {
                counter.inc();
            }
        }
        for (String key : frequency.keySet()) {
            System.out.println(key + " : " + frequency.get(key).getValue());
        }
        scanner.close();
    }

    public void splitTrainingFile() {
        PrintStream ps = null;
        // for each user creates the training file
        for (String key : frequency.keySet()) {
            // creates the output file
            try {
                ps = new PrintStream("user" + key + "_train.arff");
                for (String s : header) {
                    ps.println(s);
                }
            } catch (FileNotFoundException ex) {
                Logger.getLogger(MainBalabitTrainSplit.class.getName()).log(Level.SEVERE, null, ex);
            }
            // select rows for the give user (having id equal to key)
            selectInstances(key, ps);
            ps.close();
        }
    }

    private void selectInstances(String key, PrintStream ps) {
        Scanner scanner = null;
        try {
            scanner = new Scanner(new File(trainfile));
        } catch (FileNotFoundException ex) {
            Logger.getLogger(MainBalabitTrainSplit.class.getName()).log(Level.SEVERE, null, ex);
        }
        // ignore header
        scanner.nextLine();
        int numPositives = frequency.get(key).getValue();
        int numNegativesPerClass = numPositives / 9;
        Map<String, MyInteger> negFrequency = new HashMap<>();
        for (String k : frequency.keySet()) {
            if (k.equals(key)) {
                continue;
            }
            negFrequency.put(k, new MyInteger());
        }
        while (scanner.hasNextLine()) {
            String line = scanner.nextLine();
            String items[] = line.split(",");
            int n = items.length;
            String classid = items[n - 4].trim();
            if (classid.equals(key)) {
                // positive sample
                StringBuilder sb = new StringBuilder();
                for (int i = 0; i < n - 4; ++i) {
                    sb.append(items[i] + ",");
                }
                items[n - 5] = "1";
                sb.append(items[n - 5]);
                ps.println(sb.toString());
            } else {
                int counter = negFrequency.get(classid).getValue();
                if (counter < numNegativesPerClass) {
                    // negative sample
                    StringBuilder sb = new StringBuilder();
                    for (int i = 0; i < n - 4; ++i) {
                        sb.append(items[i] + ",");
                    }
                    items[n - 5] = "0";
                    sb.append(items[n - 5]);
                    ps.println(sb.toString());
                    negFrequency.get(classid).inc();
                }
            }
        }
        scanner.close();
    }
    
    public ArrayList<String> readHeader(String filename) {
        ArrayList<String> header = new ArrayList<>();
        Scanner scanner = null;
        try {
            scanner = new Scanner(new File(filename));

        } catch (FileNotFoundException ex) {
            Logger.getLogger(MainBalabitTrainSplit.class
                    .getName()).log(Level.SEVERE, null, ex);
        }

        while (scanner.hasNextLine()) {
            header.add(scanner.nextLine());
        }
        return header;
    }

    private static class MyInteger {
        private int value;
        
        public int getValue() {
            return value;
        }

        public void inc() {
            ++value;
        }
    }

}
